// dbg_wrapper_time_t.hpp 
//
//////////////////////////////////////////////////////////////////////

// Debug Utility library
//
// Copyright (C) 2003 John Torjo (john@torjo.com)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.
 
// See http://www.boost.org for updates, documentation, and revision history.

#if !defined(BOOST_DBG_WRAPPER_TIME_T_HPP)
#define BOOST_DBG_WRAPPER_TIME_T_HPP

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <boost/dbgutil/dbg_wrapper.hpp>
#include <time.h>
#include <iomanip>


namespace boost { namespace dbgutil {

    /* 
        specify the handle class

        Using the type directly (for instance, time_t, directly)
        won't work, since we might have multiple handle types 
        that map to the same underlying type.

        For instance, HWND and HINTERNET are different types of handles,
        but they both map to 'void *'.
    */
    struct time_t_traits {
        typedef time_t raw_type;
    };


    template<>
    struct dbg_wrapper_traits< time_t_traits, char> 
        : public dbg_wrapper_traits_base< time_t_traits, char> {

        /* 
            writes ONLY the details about the value
        */
        template< class other_char_type, class traits_type>
        static void write_details( 
                handle val, 
                std::basic_ostream< other_char_type, traits_type> & out) {

            const char * months[] =
            { "jan", "feb", "mar", "apr", "may", "jun", "jul", "aug", "sep", "oct", "nov", "dec" };

            // the details
            //
            // note: we write them as a WORD (that is, no spaces),
            //       so that when they are read from the stream, 
            //       they can just be read as a whole word
            //       (keeping it simple)
            if ( val >= 0) {
                tm details = *localtime( &val);
                out << std::setw(4) << std::setfill('0') << details.tm_year + 1900
                    << '_' 
                    << std::setw(3) << std::setfill('0') << months[ details.tm_mon]
                    << '_' 
                    << std::setw(2) << std::setfill('0') << details.tm_mday
                    << "___" 
                    << std::setw(2) << std::setfill('0') << details.tm_hour
                    << ':' 
                    << std::setw(2) << std::setfill('0') << details.tm_min 
                    << ':' 
                    << std::setw(2) << std::setfill('0') << details.tm_sec;
            }
            else {
                out << "invalid_value";
            }        
        }

        /*
            read the value (ignoring the details, if any)
        */
        template< class other_char_type, class traits_type>
        static void read_val( 
                handle &val, 
                std::basic_istream< other_char_type, traits_type> & in) {
            char c = 0;
            in >> c;
            if ( in) {
                if ( c == prefix() ) {
                    // contains details as well (which we ignore)
                    std::string ignore;
                    in >> val >> ignore;
                }
                else {
                    // no details
                    in.putback( c);
                    in >> val;
                }
            }
        }

    };


}} // namespaces



#endif // !defined(AFX_TIME_T_WRAPPER_H__5ECB1ADD_9D17_41A3_8E21_5DF960B7646A__INCLUDED_)
