// dbg_wrapper_timeduration.hpp 
//
//////////////////////////////////////////////////////////////////////

// Debug Utility library
//
// Copyright (C) 2003 John Torjo (john@torjo.com)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.
 
// See http://www.boost.org for updates, documentation, and revision history.

#if !defined(BOOST_DBG_WRAPPER_TIMEDURATION_HPP)
#define BOOST_DBG_WRAPPER_TIMEDURATION_HPP

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <boost/date_time/posix_time/posix_time_types.hpp>
#include <iomanip>


namespace boost { namespace dbgutil {

    /* 
        specify the handle class

        Using the type directly (for instance, time_t, directly)
        won't work, since we might have multiple handle types 
        that map to the same underlying type.

        For instance, HWND and HINTERNET are different types of handles,
        but they both map to 'void *'.
    */
    struct time_duration_traits {
        typedef ::boost::posix_time::time_duration raw_type;
    };

    template<>
    struct dbg_wrapper_traits< time_duration_traits, char> 
        : public dbg_wrapper_traits_base< time_duration_traits, char> {

        /* 
            writes ONLY the details about the value
        */
        template< class other_char_type, class traits_type>
        static void write_details( 
                handle val, 
                std::basic_ostream< other_char_type, traits_type> & out) {

            // the details
            //
            // note: we write them as a WORD (that is, no spaces),
            //       so that when they are read from the stream, 
            //       they can just be read as a whole word
            //       (keeping it simple)
            if ( val.is_negative() ) {
                out << '-';
            }
            out << std::setw(2) << std::setfill('0') << val.hours() << ':'
                << std::setw(2) << std::setfill('0') << val.minutes() << ':'
                << std::setw(2) << std::setfill('0') << val.seconds() << '.'
                << std::setw(9) << std::setfill('0') << (long)val.fractional_seconds();
        }


        /* 
            writes ONLY the value (no details about it)
        */
        template< class other_char_type, class traits_type>
        static void write_val( handle val, std::basic_ostream< other_char_type, traits_type> & out) {
            // note: this could be improved A LOT
            // just by reading/writing the internal value !!!

            // two values: high & low
            boost::int64_t val_ticks = val.ticks();
            unsigned long high = ( val_ticks >> 32), low = val_ticks;
            out << high << ' ' << low;
        }

        /*
            read the value (ignoring the details, if any)
        */
        template< class other_char_type, class traits_type>
        static void read_val( 
                handle &val, 
                std::basic_istream< other_char_type, traits_type> & in) {
            // note: this could be improved A LOT
            // just by reading/writing the internal value !!!
            char c = 0;
            in >> c;

            // two values: high & low
            unsigned long high = 0, low = 0;
            if ( in) {
                if ( c == prefix() ) {
                    // contains details as well (which we ignore)
                    std::string ignore;
                    in >> high >> low >> ignore;
                }
                else {
                    // no details
                    in.putback( c);
                    in >> high >> low;
                }
            }
            boost::int64_t val_ticks = high;
            val_ticks <<= 32;
            val_ticks += low;

            // ignore nanoseconds
            static const long second_ticks = ::boost::posix_time::seconds( 1).ticks();
            val = ::boost::posix_time::seconds( val_ticks / second_ticks);
        }

    };


}} // namespaces



#endif // !defined(AFX_TIME_T_WRAPPER_H__5ECB1ADD_9D17_41A3_8E21_5DF960B7646A__INCLUDED_)
