// dbgutil.cpp : Defines the entry point for the console application.
//


#include <assert.h>
#include <boost/dbgutil/dbg_wrapper_time_t.hpp>
#include <boost/dbgutil/dbg_wrapper_ptime.hpp>
#include <iostream>
using namespace boost::dbgutil;

void test_time_t() {
    // default wrapper - in debug, keeps details as well
    //                 - in release, as efficient as time_t
    //                   (except for I/O, slightly less efficient)
    typedef dbg_wrapper< time_t_traits> time_t_wrapper;
    // as efficient as time_t
    // (except for I/O, slightly less efficient)
    typedef dbg_wrapper< time_t_traits, wrapper_release> time_t_wrapper_release;


    time_t_wrapper t = time(0);
    time_t_wrapper::write_details() = false;
    // writes current time - as a time_t value
    std::cout << t << std::endl;
    time_t_wrapper::write_details() = true;
    // writes current time - with user-friendly details
    std::cout << t << std::endl;

    time_t_wrapper_release t2;
    t2 = time(0);
    time_t_wrapper_release::write_details() = false;
    // writes current time - as a time_t value
    // (this is 'release' wrapper)
    std::cout << t2 << std::endl;
    time_t_wrapper_release::write_details() = true;
    // writes current time - with user-friendly details
    // (this is 'release' wrapper)
    std::cout << t2 << std::endl;


    {
    char ch = 0;
    // testing:
    // write time_t + details, and then read it back
    time_t_wrapper::write_details() = true;
    std::ostringstream out;
    out << t << " c";
    std::istringstream in( out.str());
    time_t_wrapper_release t3;
    in >> t3 >> ch;
    assert( ( t == t3) && ( ch == 'c'));
    }

    {
    char ch = 0;
    // testing:
    // write time_t without details, and then read it back
    time_t_wrapper::write_details() = false;
    std::ostringstream out;
    out << t << " c";
    std::istringstream in( out.str());
    time_t_wrapper_release t3;
    in >> t3 >> ch;
    assert( ( t == t3) && ( ch == 'c'));
    }
}

void test_ptime() {
    // default wrapper - in debug, keeps details as well
    //                 - in release, as efficient as ptime
    //                   (except for I/O, slightly less efficient)
    typedef dbg_wrapper< ptime_traits> ptime_wrapper;
    // as efficient as ptime
    // (except for I/O, slightly less efficient)
    typedef dbg_wrapper< ptime_traits, wrapper_release> ptime_wrapper_release;

    using namespace boost::posix_time;

    ptime_wrapper t = second_clock::local_time();
    ptime_wrapper::write_details() = false;
    // writes current time - as a ptime value
    std::cout << t << std::endl;
    ptime_wrapper::write_details() = true;
    // writes current time - with user-friendly details
    std::cout << t << std::endl;

    ptime_wrapper_release t2 = second_clock::local_time();
    ptime_wrapper_release::write_details() = false;
    // writes current time - as a ptime value
    // (this is 'release' wrapper)
    std::cout << t2 << std::endl;
    ptime_wrapper_release::write_details() = true;
    // writes current time - with user-friendly details
    // (this is 'release' wrapper)
    std::cout << t2 << std::endl;


    {
    char ch = 0;
    // testing:
    // write ptime + details, and then read it back
    ptime_wrapper::write_details() = true;
    std::ostringstream out;
    out << t << " c";
    std::istringstream in( out.str());
    ptime_wrapper_release t3 = t2;
    in >> t3 >> ch;
    // if I'm not using (ptime)t, VC6 gives a compile-time error
    assert( ( (ptime)t == t3) && ( ch == 'c'));
    }

    {
    char ch = 0;
    // testing:
    // write ptime without details, and then read it back
    ptime_wrapper::write_details() = false;
    std::ostringstream out;
    out << t << " c";
    std::istringstream in( out.str());
    ptime_wrapper_release t3 = t2;
    in >> t3 >> ch;
    // if I'm not using (ptime)t, VC6 gives a compile-time error
    assert( ( (ptime)t == t3) && ( ch == 'c'));
    }
}




int main(int argc, char* argv[])
{
    test_time_t();
    test_ptime();

    std::cin.get();
	return 0;
}

